#!/usr/bin/env python  
#encoding: utf-8
import ctypes
 
STD_INPUT_HANDLE = -10
STD_OUTPUT_HANDLE= -11
STD_ERROR_HANDLE = -12
 
FOREGROUND_BLACK = 0x0
FOREGROUND_BLUE = 0x01 # text color contains blue.
FOREGROUND_GREEN= 0x02 # text color contains green.
FOREGROUND_RED = 0x04 # text color contains red.
FOREGROUND_INTENSITY = 0x08 # text color is intensified.
 
BACKGROUND_BLUE = 0x10 # background color contains blue.
BACKGROUND_GREEN= 0x20 # background color contains green.
BACKGROUND_RED = 0x40 # background color contains red.
BACKGROUND_INTENSITY = 0x80 # background color is intensified.
 
class Color:
    ''' See http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winprog/winprog/windows_api_reference.asp
    for information on Windows APIs.'''
    std_out_handle = ctypes.windll.kernel32.GetStdHandle(STD_OUTPUT_HANDLE)
    
    def set_cmd_color(self, color, handle=std_out_handle):
        """(color) -> bit
        Example: set_cmd_color(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY)
        """
        bool = ctypes.windll.kernel32.SetConsoleTextAttribute(handle, color)
        return bool
    
    def reset_color(self):
        self.set_cmd_color(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE)
    
    def print_text(self, print_text, end='\n'):
        self.set_cmd_color(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE | FOREGROUND_INTENSITY)
        print(print_text, end=end)
        self.reset_color()
    
    def print_red_text(self, print_text, end='\n'):
        self.set_cmd_color(FOREGROUND_RED | FOREGROUND_INTENSITY)
        print(print_text, end=end)
        self.reset_color()
        
    def print_green_text(self, print_text, end='\n'):
        self.set_cmd_color(FOREGROUND_GREEN | FOREGROUND_INTENSITY)
        print(print_text, end=end)
        self.reset_color()
    
    def print_blue_text(self, print_text, end='\n'): 
        self.set_cmd_color(FOREGROUND_BLUE | FOREGROUND_INTENSITY)
        print(print_text, end=end)
        self.reset_color()
    
    def print_yellow_text(self, print_text, end='\n'): 
        self.set_cmd_color(FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_INTENSITY)
        print(print_text, end=end)
        self.reset_color()
          
    def print_red_text_with_blue_bg(self, print_text, end='\n'):
        self.set_cmd_color(FOREGROUND_RED | FOREGROUND_INTENSITY| BACKGROUND_BLUE | BACKGROUND_INTENSITY)
        print(print_text, end=end)
        self.reset_color()    
 
if __name__ == "__main__":
    clr = Color()
    clr.print_red_text('red')
    clr.print_green_text('green')
    clr.print_blue_text('blue')
    clr.print_red_text_with_blue_bg('background')



# -*- coding: utf-8 -*-

###########################
# Excel数据校验
# peakgao 2018.10.23
###########################

import os
import xlrd
from .common import FieldType,FieldInfo,getTypeByName,getCellName

def ExcelVerify(srcFile, strSeparator = '|'):
    strCellName = ''
    try:    
        srcFile = os.path.abspath(srcFile)
        if not os.path.isfile(srcFile):
            return 0, "Missing '" + srcFile + "'"

        book = xlrd.open_workbook(srcFile)
        sheet = book.sheet_by_index(0)
        nrows = sheet.nrows
        ncols = sheet.ncols

        if nrows == 0 or ncols == 0: # 空文件
            return 0, "Not data from '" + srcFile + "'"

        check_head = True # 是否在扫描头部信息区
        fieldList = [] # 记录有效的字段信息列表，无效的都忽略了

        idx_desc = -1 # 字段中文描述行，一个#
        idx_verify = -1 # 字段数据校验行，两个#
        idx_cstype = -1 # c/s归类类型行，三个#
        idx_name = -1 # 字段名
        idx_type = -1 # 字段类型
        idx_data = -1 # 数据行
        
        # 扫描数据头信息
        for row in range(nrows):
            if check_head: # 头部区域
                strCellName = getCellName(row, 0)
                col0_str = sheet.cell_value(row, 0)
                if col0_str.find('###') == 0:
                    idx_cstype = row
                    continue
                elif col0_str.find('##') == 0:
                    idx_verify = row
                    continue
                elif col0_str.find('#') == 0:
                    idx_desc = row
                    continue
                elif idx_name == -1:
                    idx_name = row
                    continue
                elif idx_type == -1:
                    idx_type = row
                    idx_data = row + 1
                    check_head = False
                    #csType = csType.upper()

                    # 综合校验所有列
                    for col in range(ncols):
                        strCellName = getCellName(row, col)
                        strType = sheet.cell_value(row, col)
                        fType = getTypeByName(strType)
                        fName = sheet.cell_value(idx_name, col)
                        if fType != None and fName != '': # 有效的列，必须类型有效且名字不为空
                            isValidField = True
                            if idx_cstype != -1: # 存在CS信息，则进行类型匹配处理
                                strCSType = sheet.cell_value(idx_cstype, col).strip('#')
                            else: # 不存在CS信息，则当所有列满足条件
                                strCSType = "CS"
                            
                            if isValidField: # 有效字段，记录下来
                                fieldInfo = FieldInfo()
                                fieldInfo.col = col
                                if idx_desc >= 0:
                                    fieldInfo.desc = sheet.cell_value(idx_desc, col).strip('#')
                                if idx_verify >= 0:
                                    fieldInfo.verify = sheet.cell_value(idx_verify, col).strip('#')
                                fieldInfo.cstype = strCSType
                                fieldInfo.name = fName
                                fieldInfo.type = fType
                                fieldList.append(fieldInfo)
                        else:
                            if fType == None and strType != '':
                                return 0, "[" + strCellName + "] Type `" + strType + "` not implement"
                            if fType != None and fName == '':
                                return 0, "[" + chr(65 + col) + str(row) + "] Missing field name"

                    break
        # 处理数据    
        n = len(fieldList)
        for k in range(n):
            fieldInfo = fieldList[k]
            col = fieldInfo.col           
            fieldType = fieldInfo.type				
            strCode = fieldInfo.verify

            for row in range(idx_data, nrows):
                strCellName = getCellName(row, col)
                val = sheet.cell_value(row, col)
                
                if isinstance(val, float) and (val % 1 == 0): # Excel的整数读出来都是浮点，小数位为0，奇特，所以做下面处理
                    val = int(val)
                strVal = str(val)
                #if isinstance(strVal, str):
                strVal = strVal.strip() # 裁剪前后空白字符
                strVal = strVal.replace("\\", "\\\\") # 英文反斜杠要转义
                strVal = strVal.replace("\"", "\\\"") # 英文双引号要转义

                if fieldType == FieldType.Byte:
                    v = int(strVal)
                    innerCode = "v>=0 and v<=255"
                    if not eval(innerCode): print('[%s] verify fail with inner code [%s]' % (strCellName, innerCode))
                elif fieldType == FieldType.Short:
                    v = int(strVal)
                    innerCode = "v>=-32768 and v<=32767"
                    if not eval(innerCode): print('[%s] verify fail with inner code [%s]' % (strCellName, innerCode))
                elif fieldType == FieldType.Int or fieldType == FieldType.Index:
                    v = int(strVal)
                elif fieldType == FieldType.Float:
                    v = float(strVal)
                elif fieldType == FieldType.ByteArray:
                    strVal = strVal.strip(strSeparator) # 裁剪前后分隔符
                    v = strVal.split(strSeparator)
                    for i in range(len(v)):
                        v[i] = int(v[i])
                    innerCode = "v[i]>=0 and v[i]<=255"
                    for i in range(len(v)):
                        if not eval(innerCode): print('[%s:%d] verify fail with inner code [%s]' % (strCellName, i, innerCode))
                        if strCode != '' and not eval(strCode): print('[%s] verify fail with code [%s]' % (strCellName, strCode))
                    strCode = ''
                elif fieldType == FieldType.ShortArray:
                    strVal = strVal.strip(strSeparator) # 裁剪前后分隔符
                    v = strVal.split(strSeparator)
                    for i in range(len(v)):
                        v[i] = int(v[i])
                    innerCode = "v[i]>=-32768 and v[i]<=32767"
                    for i in range(len(v)):
                        if not eval(innerCode): print('[%s:%d] verify fail with inner code [%s]' % (strCellName, i, innerCode))
                        if strCode != '' and not eval(strCode): print('[%s] verify fail with code [%s]' % (strCellName, strCode))
                    strCode = ''
                elif fieldType == FieldType.IntArray:
                    strVal = strVal.strip(strSeparator) # 裁剪前后分隔符
                    v = strVal.split(strSeparator)
                    for i in range(len(v)):
                        v[i] = int(v[i])
                    for i in range(len(v)):
                        if strCode != '' and not eval(strCode): print('[%s] verify fail with code [%s]' % (strCellName, strCode))
                    strCode = ''
                elif fieldType == FieldType.FloatArray:
                    strVal = strVal.strip(strSeparator) # 裁剪前后分隔符
                    v = strVal.split(strSeparator)
                    for i in range(len(v)):
                        v[i] = float(v[i])
                    for i in range(len(v)):
                        if strCode != '' and not eval(strCode): print('[%s] verify fail with code [%s]' % (strCellName, strCode))
                    strCode = ''
                elif (fieldType == FieldType.Key or 
                    fieldType == FieldType.Remark or
                    fieldType == FieldType.Map or
                    fieldType == FieldType.String or
                    fieldType == FieldType.NString):
                    v = strVal
                else:
                    pass
                
                if strCode != '' and not eval(strCode): print('[%s] verify fail with code [%s]' % (strCellName, strCode))

    except BaseException as err:
        if strCellName != '':
            return -1, '[' + strCellName + '] ' + str(err)
        else:
            return -1, str(err)
            
    return 1, "OK"


# -*- coding: utf-8 -*-

###########################
# Excel转成bcc格式文本(用于C#读取的二进制文件)
# peakgao 2018.10.20
###########################

import os
import xlrd
import struct
from io import BytesIO
from .common import FieldType,FieldInfo,getTypeByName,getCellName,getFieldSize,ensureDirExist

def ExcelToBcc(srcFile, dstFile, csType, hasGT, strSeparator):
    strCellName = ''
    try:    
        srcFile = os.path.abspath(srcFile)
        if not os.path.isfile(srcFile):
            return 0, "Missing '" + srcFile + "'"

        book = xlrd.open_workbook(srcFile)
        sheet = book.sheet_by_index(0)
        nrows = sheet.nrows
        ncols = sheet.ncols

        if nrows == 0 or ncols == 0: # 空文件
            return 0, "Not data from '" + srcFile + "'"

        check_head = True # 是否在扫描头部信息区
        fieldList = [] # 记录有效的字段信息列表，无效的都忽略了

        idx_desc = -1 # 字段中文描述行，一个#
        idx_verify = -1 # 字段数据校验行，两个#
        idx_cstype = -1 # c/s归类类型行，三个#
        idx_name = -1 # 字段名
        idx_type = -1 # 字段类型
        idx_data = -1 # 数据行

        isEmpty = False
        fields = 0
        records = 0
        recordSize = 0
        
        
        # 扫描数据头信息
        for row in range(nrows):
            if check_head: # 头部区域
                strCellName = getCellName(row, 0)
                col0_str = sheet.cell_value(row, 0)
                if col0_str.find('###') == 0:
                    idx_cstype = row
                    continue
                elif col0_str.find('##') == 0:
                    idx_verify = row
                    continue
                elif col0_str.find('#') == 0:
                    idx_desc = row
                    continue
                elif idx_name == -1:
                    idx_name = row
                    continue
                elif idx_type == -1:
                    idx_type = row
                    idx_data = row + 1
                    check_head = False
                    csType = csType.upper()

                    # 综合校验所有列
                    for col in range(ncols):
                        strCellName = getCellName(row, col)
                        fType = getTypeByName(sheet.cell_value(row, col))
                        fName = sheet.cell_value(idx_name, col)
                        if fType != None and fName != '': # 有效的列，必须类型有效且名字不为空
                            isValidField = True
                            if idx_cstype != -1: # 存在CS信息，则进行类型匹配处理
                                strCSType = sheet.cell_value(idx_cstype, col).strip('#')
                                if strCSType == '' or strCSType.upper().find(csType) < 0:
                                    isValidField = False
                                else:
                                    isValidField = True
                            else: # 不存在CS信息，则当所有列满足条件
                                strCSType = csType
                                isValidField = True
                            
                            if isValidField: # 有效字段，记录下来
                                fieldInfo = FieldInfo()
                                fieldInfo.col = col
                                if idx_desc >= 0:
                                    fieldInfo.desc = sheet.cell_value(idx_desc, col).strip('#')
                                if idx_verify >= 0:
                                    fieldInfo.verify = sheet.cell_value(idx_verify, col).strip('#')
                                fieldInfo.cstype = strCSType
                                fieldInfo.name = fName
                                fieldInfo.type = fType
                                fieldList.append(fieldInfo)

                                recordSize += getFieldSize(fieldInfo.type)
                    break

        # 处理数据
        ensureDirExist(os.path.dirname(dstFile))
        fields = len(fieldList)
        records = nrows - idx_data
        minId = 0xFFFFFFFF
        maxId = 0
        f1 = BytesIO()
        f2 = BytesIO()
        f2.write(struct.pack('I', 0)) # 变长数据区前4个字节置为0，可用于空字符串和空数组的处理

        for row in range(idx_data, nrows):
            n = len(fieldList)
            for k in range(n):
                fieldInfo = fieldList[k]
                col = fieldInfo.col                
                fieldType = fieldInfo.type
                strCellName = getCellName(row, col)
                val = sheet.cell_value(row, col)
                
                if isinstance(val, float) and (val % 1 == 0): # Excel的整数读出来都是浮点，小数位为0，奇特，所以做下面处理
                    val = int(val)
                strVal = str(val)
                #if isinstance(strVal, str):
                strVal = strVal.strip() # 裁剪前后空白字符
                #strVal = strVal.replace("\\", "\\\\") # 英文反斜杠要转义
                #strVal = strVal.replace("\"", "\\\"") # 英文双引号要转义

                if len(strVal) == 0:
                    isEmpty = True
                else:
                    isEmpty = False

                if fieldType == FieldType.Byte:
                    f1.write(struct.pack('B', int(strVal)))
                elif fieldType == FieldType.Short:
                    f1.write(struct.pack('h', int(strVal)))
                elif fieldType == FieldType.Int:
                    f1.write(struct.pack('i', int(strVal)))
                elif fieldType == FieldType.Index or fieldType == FieldType.Key:
                    v = int(strVal)
                    f1.write(struct.pack('I', v))
                    if v > maxId: maxId = v
                    if v < minId: minId = v
                elif fieldType == FieldType.Float:
                    f1.write(struct.pack('f', float(strVal)))
                elif (fieldType == FieldType.Map or
                    fieldType == FieldType.String or
                    fieldType == FieldType.NString):
                    if isEmpty:
                        f1.write(struct.pack('I', 0))
                    else:
                        f1.write(struct.pack('I', f2.tell()))
                        u8 = strVal.encode('utf-8')
                        f2.write(struct.pack('H', len(u8)))
                        f2.write(u8)
                elif (fieldType == FieldType.ByteArray):
                    strVal = strVal.strip(strSeparator)
                    if isEmpty:
                        f1.write(struct.pack('I', 0))
                    else:
                        f1.write(struct.pack('I', f2.tell()))
                        v = strVal.split(strSeparator)
                        f2.write(struct.pack('I', len(v)))
                        for i in range(len(v)):
                            f2.write(struct.pack('B', int(v[i])))
                elif fieldType == FieldType.ShortArray:
                    strVal = strVal.strip(strSeparator)
                    if isEmpty:
                        f1.write(struct.pack('I', 0))
                    else:
                        f1.write(struct.pack('I', f2.tell()))
                        v = strVal.split(strSeparator)
                        f2.write(struct.pack('I', len(v)))
                        for i in range(len(v)):
                            f2.write(struct.pack('h', int(v[i])))
                elif fieldType == FieldType.IntArray:
                    strVal = strVal.strip(strSeparator) # 裁剪前后分隔符
                    if isEmpty:
                        f1.write(struct.pack('I', 0))
                    else:
                        f1.write(struct.pack('I', f2.tell()))
                        v = strVal.split(strSeparator)
                        f2.write(struct.pack('I', len(v)))
                        for i in range(len(v)):
                            f2.write(struct.pack('i', int(v[i])))
                elif fieldType == FieldType.FloatArray:
                    strVal = strVal.strip(strSeparator) # 裁剪前后分隔符
                    if isEmpty:
                        f1.write(struct.pack('I', 0))
                    else:
                        f1.write(struct.pack('I', f2.tell()))
                        v = strVal.split(strSeparator)
                        f2.write(struct.pack('I', len(v)))
                        for i in range(len(v)):
                            f2.write(struct.pack('f', float(v[i])))
                elif fieldType == FieldType.Remark:
                    f1.write(struct.pack('I', 0))
        
        strCellName = ''
        ID = 0x00CCBBDD
        f = open(dstFile, 'wb')
        f.write(struct.pack('I', ID))
        f.write(struct.pack('I', records))
        f.write(struct.pack('I', fields))
        f.write(struct.pack('I', recordSize))
        f.write(struct.pack('I', f2.tell())) # 变长缓冲区大小
        f.write(struct.pack('I', minId))
        f.write(struct.pack('I', maxId))
        # 写入字段列表
        n = len(fieldList)
        for k in range(n):
            fieldInfo = fieldList[k]
            f.write(struct.pack('B', fieldInfo.type.value))

        # 写定长数据
        f.write(f1.getvalue())
        f1.close()

        # 写变长数据
        f.write(f2.getvalue())
        f2.close()

        # 关闭文件
        f.close()

    except BaseException as err:
        if strCellName != '':
            return -1, '[' + strCellName + '] ' + str(err)
        else:
            return -1, str(err)

    return 1, "OK"



# -*- coding: utf-8 -*-

#####################################
# Excel将数据结构转成c#格式的文件
# peakgao 2018.10.24
#####################################

import os
import xlrd
from .common import FieldType,FieldInfo,getTypeByName,getCellName,getFieldSize,getFuncName,getTypeNameForCs,saveUtf8File,ensureDirExist

def ExcelToCs(srcFile, dstFile, csType):
    strCellName = ''
    try:
        srcFile = os.path.abspath(srcFile)
        if not os.path.isfile(srcFile):
            return 0, "Missing '" + srcFile + "'"

        book = xlrd.open_workbook(srcFile)
        sheet = book.sheet_by_index(0)
        nrows = sheet.nrows
        ncols = sheet.ncols

        if nrows == 0 or ncols == 0: # 空文件
            return 0, "Not data from '" + srcFile + "'"

        check_head = True # 是否在扫描头部信息区
        fieldList = [] # 记录有效的字段信息列表，无效的都忽略了

        idx_desc = -1 # 字段中文描述行，一个#
        idx_verify = -1 # 字段数据校验行，两个#
        idx_cstype = -1 # c/s归类类型行，三个#
        idx_name = -1 # 字段名
        idx_type = -1 # 字段类型
        idx_data = -1 # 数据行

        out = "" # 输出文本
        recordSize = 0
        maxNameLength = 0
        maxTypeLength = 0

        # 扫描数据头信息
        for row in range(nrows):
            if check_head: # 头部区域
                strCellName = getCellName(row, 0)
                col0_str = sheet.cell_value(row, 0)
                if col0_str.find('###') == 0:
                    idx_cstype = row
                    continue
                elif col0_str.find('##') == 0:
                    idx_verify = row
                    continue
                elif col0_str.find('#') == 0:
                    idx_desc = row
                    continue
                elif idx_name == -1:
                    idx_name = row
                    continue
                elif idx_type == -1:
                    idx_type = row
                    idx_data = row + 1
                    check_head = False
                    csType = csType.upper()

                    # 综合校验所有列
                    for col in range(ncols):
                        strCellName = getCellName(row, col)
                        fType = getTypeByName(sheet.cell_value(row, col))
                        fName = sheet.cell_value(idx_name, col)
                        if fType != None and fName != '': # 有效的列，必须类型有效且名字不为空
                            isValidField = True
                            if idx_cstype != -1: # 存在CS信息，则进行类型匹配处理
                                strCSType = sheet.cell_value(idx_cstype, col).strip('#')
                                if strCSType == '' or strCSType.upper().find(csType) < 0:
                                    isValidField = False
                                else:
                                    isValidField = True
                            else: # 不存在CS信息，则当所有列满足条件
                                strCSType = csType
                                isValidField = True
                            
                            if isValidField: # 有效字段，记录下来
                                fieldInfo = FieldInfo()
                                fieldInfo.col = col
                                if idx_desc >= 0:
                                    fieldInfo.desc = sheet.cell_value(idx_desc, col).strip('#')
                                if idx_verify >= 0:
                                    fieldInfo.verify = sheet.cell_value(idx_verify, col).strip('#')
                                fieldInfo.cstype = strCSType
                                fieldInfo.name = fName
                                fieldInfo.type = fType
                                fieldList.append(fieldInfo)
                                
                                recordSize += getFieldSize(fieldInfo.type)
                                if len(fieldInfo.name) > maxNameLength:
                                    maxNameLength = len(fieldInfo.name)
                                type_len = len(getTypeNameForCs(fType))
                                if type_len > maxTypeLength:
                                    maxTypeLength = type_len
                    break

        # 处理数据

        # 写表名
        # tableName = sheet.name[0].upper() + sheet.name[1:].lower()
        tableName = sheet.name.capitalize()

        out += "// This file had generated by Python, NOT MODIFY\nusing Core;\n"
        out += "namespace Config." + tableName + "\n{\n"
        out += "    public class Entry : IEntry\n    {\n"

        # 处理字段名列表            
        n = len(fieldList)
        maxTypeLength = (maxTypeLength+3) & ~3
        maxNameLength = (maxNameLength+1+3) & ~3
        for i in range(n):
            fieldInfo = fieldList[i]
            out += "        public " + getTypeNameForCs(fieldInfo.type).ljust(maxTypeLength) + (fieldInfo.name + ";").ljust(maxNameLength)
            if fieldInfo.desc != '':
                out += "// " + fieldInfo.desc
            out += "\n"
        out += "    }\n\n"
        
        # 写类代码
        out += "    public class Data : BCCFile\n"
        out += "    {\n"
        out += "        public override int GetRecordSize() { return " + str(recordSize) + "; }\n"
        out += "        public override IEntry CreateEntry() { return new Entry(); }\n"
        out += "        public override void UpdateEntry(IEntry entry)\n"
        out += "        {\n"
        out += "            Entry en = entry as Entry;\n"
        
        
        for i in range(n):
            fieldInfo = fieldList[i]
            out += "            en." + fieldInfo.name + "=" + getFuncName(fieldInfo.type) + "();\n"
        
        if fieldList[0].type == FieldType.Index:
            out += "            AddIndex(en." + sheet.cell_value(idx_name, 0) + ", en);\n"
        elif fieldList[0].type == FieldType.Key:
            out += "            AddKey(en." + sheet.cell_value(idx_name, 0) + ", en);\n"
        elif fieldList[0].type == FieldType.Map:
            out += "            AddMap(en." + sheet.cell_value(idx_name, 0) + ", en);\n"

        out += "        }\n"
        out += "    }\n"
        out += "}\n"

        ensureDirExist(os.path.dirname(dstFile))
        saveUtf8File(dstFile, out)

    except BaseException as err:
        if strCellName != '':
            return -1, '[' + strCellName + '] ' + str(err)
        else:
            return -1, str(err)

    return 1, "OK"



# -*- coding: utf-8 -*-

###########################
# 公共的枚举值定义
# peakgao 2018.10.20
###########################

from enum import Enum,unique

@unique
class FieldType(Enum):
    Byte = 0
    Short = 1
    Int = 2
    Float = 3
    ByteArray = 4
    ShortArray = 5
    IntArray = 6
    FloatArray = 7
    String = 8
    NString = 9
    Remark = 10
    Index = 11
    Key = 12
    Map = 13

class FieldInfo:
    col = -1    # 列索引，从0开始
    desc = ''   # 字段描述
    verify = '' # 字段校验代码
    cstype = '' # 字段CS类型
    name = ''   # 字段名
    type = None # 字段类型



def getTypeByName(typeName):
    typeName == typeName.lower()
    if typeName == "byte":
        return FieldType.Byte
    elif typeName == "short":
        return FieldType.Short
    elif typeName == "int":
        return FieldType.Int
    elif typeName == "float":
        return FieldType.Float
    elif typeName == "byte[]":
        return FieldType.ByteArray
    elif typeName == "short[]":
        return FieldType.ShortArray
    elif typeName == "int[]":
        return FieldType.IntArray
    elif typeName == "float[]":
        return FieldType.FloatArray
    elif typeName == "string":
        return FieldType.String
    elif typeName == "nstring":
        return FieldType.NString
    elif typeName == "remark":
        return FieldType.Remark
    elif typeName == "index":
        return FieldType.Index
    elif typeName == "key":
        return FieldType.Key
    elif typeName == "map":
        return FieldType.Map
    else:
        return None

def getFuncName(iType):
    typeName = ""    
    if iType == FieldType.Byte:
        typeName = "ReadUInt8"
    elif iType == FieldType.Short:
        typeName = "ReadInt16"
    elif iType == FieldType.Int:
        typeName = "ReadInt32"
    elif iType == FieldType.Float:
        typeName = "ReadFloat"
    elif iType == FieldType.ByteArray:
        typeName = "ReadUIntArray8"
    elif iType == FieldType.ShortArray:
        typeName = "ReadIntArray16"
    elif iType == FieldType.IntArray:
        typeName = "ReadIntArray32"
    elif iType == FieldType.FloatArray:
        typeName = "ReadFloatArray"
    elif iType == FieldType.String:
        typeName = "ReadString"
    elif iType == FieldType.NString:
        typeName = "ReadString"
    elif iType == FieldType.Remark:
        typeName = "ReadString"
    elif iType == FieldType.Key:
        typeName = "ReadInt32"
    elif iType == FieldType.Map:
        typeName = "ReadString"
    elif iType == FieldType.Index:
        typeName = "ReadInt32"
    else:
        typeName = "ReadInt32"
    return typeName

def getTypeNameForCs(iType):
    typeName = ""    
    if iType == FieldType.Byte:
        typeName = "byte"
    elif iType == FieldType.Short:
        typeName = "short"
    elif iType == FieldType.Int:
        typeName = "int"
    elif iType == FieldType.Float:
        typeName = "float"
    elif iType == FieldType.ByteArray:
        typeName = "byte[]"
    elif iType == FieldType.ShortArray:
        typeName = "short[]"
    elif iType == FieldType.IntArray:
        typeName = "int[]"
    elif iType == FieldType.FloatArray:
        typeName = "float[]"
    elif iType == FieldType.String:
        typeName = "string"
    elif iType == FieldType.NString:
        typeName = "string"
    elif iType == FieldType.Remark:
        typeName = "string"
    elif iType == FieldType.Key:
        typeName = "int"
    elif iType == FieldType.Map:
        typeName = "string"
    elif iType == FieldType.Index:
        typeName = "int"
    else:
        typeName = "int"
    return typeName


def getTypeNameForCpp(iType):
    typeName = ""    
    if iType == FieldType.Byte:
        typeName = "byte"
    elif iType == FieldType.Short:
        typeName = "short"
    elif iType == FieldType.Int:
        typeName = "int"
    elif iType == FieldType.Float:
        typeName = "float"
    elif iType == FieldType.ByteArray:
        typeName = "const I8A*"
    elif iType == FieldType.ShortArray:
        typeName = "const I16A*"
    elif iType == FieldType.IntArray:
        typeName = "const I32A*"
    elif iType == FieldType.FloatArray:
        typeName = "const F32A*"
    elif iType == FieldType.String:
        typeName = "const utf8*"
    elif iType == FieldType.NString:
        typeName = "const utf8*"
    elif iType == FieldType.Remark:
        typeName = "const utf8*"
    elif iType == FieldType.Key:
        typeName = "uint"
    elif iType == FieldType.Map:
        typeName = "const utf8*"
    elif iType == FieldType.Index:
        typeName = "uint"
    else:
        typeName = "int"
    return typeName

def getFieldSize(iFieldType):
    if iFieldType == None:
        return 0
    elif iFieldType == FieldType.Byte:
        return 1
    elif iFieldType == FieldType.Short:
        return 2
    else:
        return 4


def getCellName(row, col):
    return chr(65 + col) + str(row + 1)


def ensureDirExist(dir):
    if not os.path.isdir(dir):
        os.makedirs(dir)

import os,hashlib#,datetime

def getFileMd5(filename):
    if not os.path.isfile(filename):
        return ''
    myhash = hashlib.md5()
    f = open(filename,'rb')
    while True:
        b = f.read(8096)
        if not b :
            break
        myhash.update(b)
    f.close()
    return myhash.hexdigest()

"""
starttime = datetime.datetime.now()
print(getFileMd5("E:\\works\\nodejs\\python\\demo.xls"))
endtime = datetime.datetime.now()
print('运行时间：%ds'%((endtime-starttime).seconds))
"""

def readFileLastMd5(filename):
    s = ""
    if not os.path.isfile(filename):
        return s    
    f = open(filename)    
    try:
        s = f.read()
    finally:
        f.close()
    return s

def writeFileLastMd5(filename, strMd5):
    parent_path = os.path.dirname(filename)
    if not os.path.isdir(parent_path):
        os.makedirs(parent_path) # 递归创建目录    
    f = open(filename, 'w')    
    try:
        f.write(strMd5)
    finally:
        f.close()

def saveUtf8File(filename, contents):
    ensureDirExist(os.path.dirname(filename))
    fh = open(filename, 'w', encoding='utf-8')
    fh.write(contents)
    fh.close()


# -*- coding: utf-8 -*-

###########################
# Excel转成Ts格式文本
# peakgao 2018.10.20
###########################

import os
import xlrd
from .common import FieldType,FieldInfo,getTypeByName,getCellName,getFieldSize,getFuncName,getTypeNameForCs,saveUtf8File,ensureDirExist

def ExcelToTs(srcFile, dstFile, csType, hasGT, strSeparator):
    strCellName = ''
    try:    
        srcFile = os.path.abspath(srcFile)
        if not os.path.isfile(srcFile):
            return 0, "Missing '" + srcFile + "'"

        book = xlrd.open_workbook(srcFile)
        sheet = book.sheet_by_index(0)
        nrows = sheet.nrows
        ncols = sheet.ncols

        if nrows == 0 or ncols == 0: # 空文件
            return 0, "Not data from '" + srcFile + "'"

        check_head = True # 是否在扫描头部信息区
        fieldList = [] # 记录有效的字段信息列表，无效的都忽略了

        idx_desc = -1 # 字段中文描述行，一个#
        idx_verify = -1 # 字段数据校验行，两个#
        idx_cstype = -1 # c/s归类类型行，三个#
        idx_name = -1 # 字段名
        idx_type = -1 # 字段类型
        idx_data = -1 # 数据行

        hasIndexCol = False # 是否第一列是索引列
        hasNString = False # 是否存在中文字符串字段
        hadWriteIndex = False
        sep = ''
        isEmpty = False
        totalText = '' # 输出的字符串
        
        # 扫描数据头信息
        for row in range(nrows):
            if check_head: # 头部区域
                strCellName = getCellName(row, 0)
                col0_str = sheet.cell_value(row, 0)
                if col0_str.find('###') == 0:
                    idx_cstype = row
                    continue
                elif col0_str.find('##') == 0:
                    idx_verify = row
                    continue
                elif col0_str.find('#') == 0:
                    idx_desc = row
                    continue
                elif idx_name == -1:
                    idx_name = row
                    continue
                elif idx_type == -1:
                    idx_type = row
                    idx_data = row + 1
                    check_head = False
                    csType = csType.upper()

                    # 综合校验所有列
                    for col in range(ncols):
                        strCellName = getCellName(row, col)
                        fType = getTypeByName(sheet.cell_value(row, col))
                        fName = sheet.cell_value(idx_name, col)
                        if fType != None and fName != '': # 有效的列，必须类型有效且名字不为空
                            isValidField = True
                            if idx_cstype != -1: # 存在CS信息，则进行类型匹配处理
                                strCSType = sheet.cell_value(idx_cstype, col).strip('#')
                                if strCSType == '' or strCSType.upper().find(csType) < 0:
                                    isValidField = False
                                else:
                                    isValidField = True
                            else: # 不存在CS信息，则当所有列满足条件
                                strCSType = csType
                                isValidField = True
                            
                            if isValidField: # 有效字段，记录下来
                                fieldInfo = FieldInfo()
                                fieldInfo.col = col
                                if idx_desc >= 0:
                                    fieldInfo.desc = sheet.cell_value(idx_desc, col).strip('#')
                                if idx_verify >= 0:
                                    fieldInfo.verify = sheet.cell_value(idx_verify, col).strip('#')
                                fieldInfo.cstype = strCSType
                                fieldInfo.name = fName
                                fieldInfo.type = fType
                                fieldList.append(fieldInfo)

                                if hasNString == False and fType == FieldType.NString:
                                    hasNString = True
                    break

        # 处理数据
        # tableName = sheet.name[0].upper() + sheet.name[1:].lower()
        tableName = sheet.name.capitalize()

        if hasNString and hasGT:
            totalText += "import _GT from './getstring';\n"
        if fieldList[0].type == FieldType.Index or fieldList[0].type == FieldType.Key or fieldList[0].type == FieldType.Map:
            hasIndexCol = True
        totalText += "export default class Cfg" + tableName + " {data={\n"

        for row in range(idx_data, nrows):
            hadWriteIndex = False
            n = len(fieldList)
            for i in range(n):
                fieldInfo = fieldList[i]
                col = fieldInfo.col
                fieldType = fieldInfo.type
                strCellName = getCellName(row, col)
                strFieldName = fieldInfo.name
                val = sheet.cell_value(row, col)
                
                if isinstance(val, float) and (val % 1 == 0): # Excel的整数读出来都是浮点，小数位为0，奇特，所以做下面处理
                    val = int(val)
                strVal = str(val)
                #if isinstance(strVal, str):
                strVal = strVal.strip() # 裁剪前后空白字符
                strVal = strVal.replace("\\", "\\\\") # 英文反斜杠要转义
                strVal = strVal.replace("\"", "\\\"") # 英文双引号要转义

                if hasIndexCol:
                    if i == 0:
                        sep = ""
                    else:
                        sep = ","
                else:
                    if i == 0:
                        sep = ""
                    else:
                        sep = ","
                
                if len(strVal) == 0:
                    isEmpty = True
                else:
                    isEmpty = False
        
                if not hasIndexCol:
                    if i == 0:
                        totalText = totalText + "{"

                if fieldType == FieldType.Remark:
                    totalText = totalText + sep + strFieldName + ':""'
                elif fieldType == FieldType.Index or fieldType == FieldType.Key:
                    if hasIndexCol and (not hadWriteIndex):
                        totalText = totalText + "[" + strVal + "]:{" + sep + strFieldName + ":" + strVal
                        hadWriteIndex = True
                elif (fieldType == FieldType.Byte or 
                    fieldType == FieldType.Short or
                    fieldType == FieldType.Int or
                    fieldType == FieldType.Index or
                    fieldType == FieldType.Float or
                    fieldType == FieldType.Key):
                    if isEmpty:
                        strVal = "0"
                    totalText = totalText + sep + strFieldName + ":" + strVal
                elif fieldType == FieldType.Map:
                    if hasIndexCol and (not hadWriteIndex):
                        totalText = totalText + '"' + strVal  + '":{' + sep + strFieldName + ':"' + strVal + '"'
                        hadWriteIndex = True
                elif fieldType == FieldType.String:
                    if isEmpty:
                        totalText = totalText + sep + strFieldName + ':""'
                    else:
                        totalText = totalText + sep + strFieldName + ':"' + strVal + '"'
                elif fieldType == FieldType.NString:
                    if isEmpty:
                        totalText = totalText + sep + strFieldName + ':""'
                    elif hasGT:
                        totalText = totalText + sep + strFieldName + ':_GT("' + strVal + '")'
                    else:
                        totalText = totalText + sep + strFieldName + ':"' + strVal + '"'
                elif (fieldType == FieldType.ByteArray or 
                    fieldType == FieldType.ShortArray or
                    fieldType == FieldType.FloatArray or
                    fieldType == FieldType.IntArray):
                    strVal = strVal.strip(strSeparator) # 裁剪前后分隔符
                    if isEmpty:
                        totalText = totalText + sep + strFieldName + ":[" + strVal + "]"
                    else:
                        strVal = strVal.replace(strSeparator, ",")
                        totalText = totalText + sep + strFieldName + ":[" + strVal + "]"
                else:
                    totalText = totalText + sep + strVal
                    #print(sheet.cell_value(row, col))

            totalText = totalText + "},\n"

        totalText = totalText + "}}\n"
        ensureDirExist(os.path.dirname(dstFile))
        saveUtf8File(dstFile, totalText)

    except BaseException as err:
        if strCellName != '':
            return -1, '[' + strCellName + '] ' + str(err)
        else:
            return -1, str(err)

    return 1, "OK"



#import sys

from .common import getFileMd5,readFileLastMd5,writeFileLastMd5
from .color import Color
from .convert import ConvertExcelDataToFile,ExportExcelStructToFile,ConvertExcelDataToFileWhenChanged
from .excelverify import ExcelVerify

__all__ = (
    'Color',
    'ConvertExcelDataToFile',
    'ExportExcelStructToFile',
    'ConvertExcelDataToFileWhenChanged',
    'ExcelVerify',
)



# coding:utf-8

from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
	name='wowlib',
	version='1.5.6',
	description='Excel convert tools for wow studio',	
	long_description=long_description,
	long_description_content_type="text/markdown",
	author='peakgao',
	author_email='peakgao163@163.com',
	license="MIT",
	url='https://www.q1.com',
	packages=['wowlib'],
	#packages=setuptools.find_packages(),
	install_requires=[
        "xlrd",
        ],	
	classifiers=[
	    "Programming Language :: Python :: 3",
	    "License :: OSI Approved :: MIT License",
	    "Operating System :: OS Independent",
	],
)


# -*- coding: utf-8 -*-

###########################
# 所有的转换操作
# peakgao 2018.10.20
###########################

import os
from .common import getFileMd5,readFileLastMd5,writeFileLastMd5,ensureDirExist
from .color import Color
from .excel2ts import ExcelToTs
from .excel2lua import ExcelToLua
from .excel2cs import ExcelToCs
from .excel2bcc import ExcelToBcc
from .excel2h import ExcelToH
from .excel2dbc import ExcelToDbc

# 转换Excel文件到指定类型的数据文件
def ConvertExcelDataToFile(srcFile, dstFile, convType, csType, hasGT):
    if convType == "ts": # 文本数据,TypeScript使用
        return ExcelToTs(srcFile, dstFile, csType, hasGT, '|')
    elif convType == "bcc": # 二进制数据,C#使用
        return ExcelToBcc(srcFile, dstFile, csType, hasGT, '|')
    elif convType == "dbc": # 二进制数据,C/C++使用
        return ExcelToDbc(srcFile, dstFile, csType, hasGT, '|')
    elif convType == "lua": # 文本数据,Lua使用
        return ExcelToLua(srcFile, dstFile, csType, hasGT, '|')
    else:
        raise Exception("Unknow convert type: " + convType)

# 将Excel字段结构导出成指定类型的文件结构
def ExportExcelStructToFile(srcFile, dstFile, convType, csType):    
    if convType == "h": # 文本结构,C/C++使用
        return ExcelToH(srcFile, dstFile, csType)
    elif convType == "cs": # 文本结构,C#使用
        return ExcelToCs(srcFile, dstFile, csType)
    else:
        raise Exception("Unknow convert type: " + convType)

# 辅助方法，对原始文件有MD5码验证，只有改变了的Excel文件才进行转换
def ConvertExcelDataToFileWhenChanged(srcFile, dstFile, logFile, convType, csType, hasGT):
    errId = 0
    errMsg = ''
    try:
        #fn = os.path.basename(srcFile)
        #ft = os.path.basename(os.path.splitext(srcFile)[0])
        #dstFile = os.path.normpath(os.path.dirname(srcFile) + "\\" + convType + "\\" + convType + "_" + ft + "." + convType)
        #logFile = os.path.normpath(os.path.dirname(srcFile) + "\\log\\" + fn + "." + convType  + "." + csType + ".txt")
        
        ensureDirExist(os.path.dirname(dstFile))
        ensureDirExist(os.path.dirname(logFile))
        
        needConvert = True
        if os.path.exists(dstFile) and os.path.exists(logFile):
            lastMd5 = readFileLastMd5(logFile)
            if lastMd5 != "" and lastMd5 == getFileMd5(srcFile):
                needConvert = False
        
        if needConvert:
            ret,msg = ConvertExcelDataToFile(srcFile, dstFile, convType, csType, hasGT)
            if ret == 1:
                newMd5 = getFileMd5(srcFile)
                writeFileLastMd5(logFile, newMd5)
            errId = ret
            errMsg = msg             
        else:
            errId = 2
            errMsg = "NOT CHANGED" 

    except ModuleNotFoundError as err:
        errId = -1
        errMsg = str(err)
    except BaseException as err:
        errMsg = str(err)

    return errId,errMsg

